import os
from enum import Enum
from logging import getLogger

__version__ = "0.1.8"

logging = getLogger()
logging.propagate = True


class LinkMode(Enum):
    SKIP = "SKIP"
    OVERRIDE = "OVERRIDE"


DEFAULT_CONFIG_NAME = ".cfgcaddy.yml"

DEFAULT_DOTFILES_DIR = os.path.join(os.path.expanduser("~"), "dotfiles")

HOME_DIR = os.path.expanduser("~")
DEFAULT_CONFIG_PATH = os.path.join(HOME_DIR, DEFAULT_CONFIG_NAME)



#!/usr/bin/env python
import ctypes
import logging
import os
import platform
import sys

import click
from questionary import prompt

import cfgcaddy
import cfgcaddy.config
import cfgcaddy.utils
import cfgcaddy.linker

logger = logging.getLogger()
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.INFO)

config_questions = {
    "preferences": [
        {
            "type": "input",
            "name": "linker_src",
            "message": "Where are your config files located?",
            "validate": lambda p: p and os.path.isdir(cfgcaddy.utils.expand_path(p)),
        },
        {
            "type": "input",
            "name": "linker_dest",
            "message": "Where should your configs be linked to?",
            "default": cfgcaddy.HOME_DIR,
            "validate": lambda p: p and os.path.isdir(cfgcaddy.utils.expand_path(p)),
        }
        # TODO: Add additional preferences like what to do on a conflict
        # or the ability to use a basic ignore (.git, only .*, etc)
    ],
}

default_config = {"links": [], "ignore": []}


def create_config(config_path, new_config=None):
    logger.debug("default_config: {}".format(new_config))

    if not new_config:
        new_config = default_config

    for section, questions in config_questions.items():
        if not new_config.get(section):
            new_config[section] = prompt(questions)

    logger.debug("Creating Config => {}".format(new_config))

    config = cfgcaddy.config.LinkerConfig(
        config_file_path=config_path, default_config=new_config
    )
    config.write_config()
    return config


@click.group()
@click.option("-d", "--debug", is_flag=True, help="Enable Debugging output")
@click.option("-q", "--quiet", is_flag=True, help="Silence cfgcaddy")
def main(debug, quiet):
    """A tool for managing your configuration files"""
    if debug:
        logger.setLevel(logging.DEBUG)
    if quiet:
        logger.setLevel(logging.ERROR)


@main.command()
@click.option(
    "-c",
    "--config",
    default=cfgcaddy.DEFAULT_CONFIG_PATH,
    help="The path to your cfgcaddy.yml",
)
@click.option("-y", "--no-interactive", is_flag=True)
def link(config, no_interactive):
    """Link your config files"""
    if not os.path.isfile(config):
        logger.error(
            "Cannot find cfgcaddy.yml, please specify path to config using '-c' option or create/link a new config using the 'cfgcaddy init' command."
        )
        return
    else:
        linker_config = cfgcaddy.config.LinkerConfig(config_file_path=config)

    caddy = cfgcaddy.linker.Linker(linker_config, interactive=no_interactive)
    caddy.create_links()
    caddy.create_custom_links()


@main.command()
@click.argument("src_directory", type=click.Path(exists=True, file_okay=False))
@click.argument("dest_directory", type=click.Path(exists=True, file_okay=False))
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    help="The path to your cfgcaddy.yml",
)
def init(src_directory, dest_directory, config):
    """Create or import a caddy config"""
    src_config_path = os.path.join(src_directory, cfgcaddy.DEFAULT_CONFIG_NAME)

    if os.path.isfile(cfgcaddy.DEFAULT_CONFIG_PATH):
        logger.error("A cfgcaddy config is already present")
    elif config:
        symlink_config("provided", config, cfgcaddy.DEFAULT_CONFIG_PATH)
    elif os.path.exists(src_config_path):
        symlink_config("existing", src_config_path, cfgcaddy.DEFAULT_CONFIG_PATH)
    else:
        default_config.update(
            {
                "preferences": {
                    "linker_src": src_directory,
                    "linker_dest": dest_directory,
                }
            }
        )
        create_config(cfgcaddy.DEFAULT_CONFIG_PATH, default_config)


def symlink_config(kind, src, dest):
    try:
        os.symlink(src, dest)
        logger.info("Symlinking {} cfgcaddy config".format(kind))
    except OSError:
        logger.error("Symlinking {} cfgcaddy config failed".format(kind))
        if platform.system() == "Windows":
            logger.error(
                "Ensure that cfgcaddy is being run as an Administrator.\n"
                "By default, only administrators can create Symlinks on Windows."
            )


def is_admin():
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False


if __name__ == "__main__":
    if platform.system() == "Windows" and not is_admin():
        ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, "", None, 1)
    main(prog_name="cfgcaddy")



from __future__ import annotations
import logging
import os
from os import path
from pathlib import Path
from typing import Union, Optional, List

from questionary import prompt

logger = logging.getLogger()

Pathlike = Union[str, Path]


def user_confirm(question: str, default: bool = True) -> bool:
    """Ask the user to confirm a choice

    Args:
        question (string): a string that is presented to the user.
        default (string): the answer if the user just hits <Enter>.
            It must be True, False or None (meaning
            an answer is required of the user).

    Returns "answer" return value is True for "yes" or False for "no".
    """
    return prompt(
        [{"type": "confirm", "name": "ok", "message": question, "default": default}]
    ).get("ok", False)


def make_parent_dirs(file_path: Pathlike) -> None:
    try:
        os.makedirs(path.dirname(file_path))
    except OSError as e:  # Python >2.5
        pass


def expand_path(file_path: Pathlike) -> Path:
    return Path(os.path.expandvars(os.path.expanduser(file_path)))


def create_dirs(dirs: Optional[List[str]] = None) -> None:
    """Creates all folders in dirs

    Args:
        dirs ([string]): A list of paths to be linked

    Returns:
        None: Does not return anything
    """
    if dirs:
        for dir_name in dirs:
            try:
                os.makedirs(dir_name)
            except OSError:
                logger.error("Unable to create directory: {}", dir_name)


def convert_to_path(f: Union[str, Path]) -> Path:
    if isinstance(f, str):
        f = Path(f)
    return f



import logging
from typing import Collection

import cfgcaddy.utils as utils
from cfgcaddy.link import create_links, find_absences
from cfgcaddy.link_spec import LinkSpec, LinkingResult
from cfgcaddy.utils import create_dirs

logger = logging.getLogger()


class Linker:
    """Tyler Stapler's Config Linker"""

    custom_links: Collection[LinkSpec]

    def __init__(self, linker_config, interactive=True) -> None:

        if not linker_config:
            raise Exception("Linker requires Config!")

        self.config = linker_config
        self.interactive = interactive

        self.custom_links = self.config.links
        self.ignored_patterns = self.config.ignore_patterns

    def create_links(self) -> None:
        """Symlinks configuration files to the destination directory

        Parses the ignore file for regexes and then generates a list of files
        which need to be symlinked"""

        # TODO: Rewrite find_absences
        absent_files, absent_dirs = find_absences(
            self.config.linker_src, self.config.linker_dest, self.ignored_patterns
        )

        if not absent_files and not absent_dirs:
            logger.info("Nothing to do")
            return

        logger.info("Preparing to symlink the following files")
        logger.info("\n".join(str(link.dest) for link in absent_files))
        if not self.interactive or utils.user_confirm("Are these the correct files?"):
            create_dirs(dirs=absent_dirs)
            create_links(links=absent_files)

    def create_custom_links(self) -> None:
        """Link all the files in the customlink file

        Returns:
            None
        """
        modified = LinkingResult.SKIPPED

        for link in self.custom_links:
            modified = link.create(interactive=self.interactive)

        if not modified:
            logger.info("No folders to link")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cfgcaddy']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.0,<9.0.0',
 'pathspec>=0.8.1',
 'questionary>=1.9.0,<2.0.0',
 'ruamel.yaml>=0.16.12,<0.17.0']

entry_points = \
{'console_scripts': ['cfgcaddy = cfgcaddy.__main__:main',
                     'publish = cfgcaddy.publish:publish']}

setup_kwargs = {
    'name': 'cfgcaddy',
    'version': '0.1.8',
    'description': "A cli for managing user's config files",
    'long_description': "# Config Caddy\n\n![Travis CI](https://travis-ci.org/tstapler/cfgcaddy.svg?branch=master)\n[![PyPI version](https://badge.fury.io/py/cfgcaddy.svg)](https://badge.fury.io/py/cfgcaddy)\n\n\nConfig Caddy is a tool used for managing your configuration files. \n\nOne way to version your configuration files is to keep them in a [git](https://git-scm.com/) repository. This has several drawbacks, among them, git requires the files it manages to be located within a single file tree. You can overcome this limitation by using symbolic links. The links point from locations in your filesystem such as `/etc/someconfig.conf`to files within the git repository like `$HOME/tstapler/dotfiles/someconfig.conf`.\n\nConfig Caddy creates symlinks to your dotfiles directory, so you don't have to. Each time you add a new file to your dotfiles repo, run `cfgcaddy link` to generate symlinks. By default `cfgcaddy` will create links from files in the dotfiles repo to their relative location in your home directory. `cfgcaddy` will also read from a configuration file. This allows the user to ignore certain files and create more complex linking relationships.\n\n\n## Usage\n\n2. Install using pip\n\n```shell\n\npip install cfgcaddy\n\n````\n2. Generate or import a configuration file for cfgcaddy\n\n```shell\ncfgcaddy init --help\n\nUsage: cfgcaddy init [OPTIONS] SRC_DIRECTORY DEST_DIRECTORY\n\n  Create or import a caddy config\n\nOptions:\n  -c, --config PATH  The path to your cfgcaddy.yml\n  --help             Show this message and exit.\n\ncfgcaddy init $HOME/dotfiles $HOME\n```\n\n3. Run the linker\n```bash\ncfgcaddy link\n```\n\n## Configuration File Format\n\n`cfgcaddy` uses a configuration file to store important information about your configuration. The `.cfgcaddy.yml` consists of several parts. A *preferences* section which contains information like where your dotfiles are located and where you want them linked to, A *links* section where you can specify more complex symlinks, and an `ignore` section where you specify files you do not want managed by `cfgcaddy`.\n\nConfig files for cfgcaddy are stored in your home directory by default. `$HOME/.cfgcaddy.yml`\n\nA sample `.cfgcaddy.yml` which leverages most features can be found [here](https://github.com/tstapler/dotfiles/blob/master/.cfgcaddy.yml)\n\n## Development\n\n1. Clone the repository\n\n```shell\n\ngit clone https://github.com/tstapler/cfgcaddy\n\n```\n\n2. Install cfgcaddy as an editable package\n\n```shell\n\npip install --editable ./cfgcaddy\n\n```\n\n3. ??????\n\n4. Profit\n\n## Motivation/Prior Art\n\nI'm an automation fiend, this tool grew out of a personal need to manage my configurations across multiple machines and operating systems. It fits my needs and I'll add features as I need them (Like Windows support). PRs are always welcome :)\n\nHere are several projects that I've drawn inspiration from:\n\n- https://github.com/charlesthomas/linker\n- https://github.com/thoughtbot/rcm\n",
    'author': 'Tyler Stapler',
    'author_email': 'tystapler@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)



import logging
from enum import Enum
from pathlib import Path
from typing import Union

from cfgcaddy.utils import convert_to_path

logger = logging.getLogger()


class FileState(str, Enum):
    MISSING = "MISSING"
    LINK = "LINK"
    BROKEN_LINK = "BROKEN_LINK"
    LINK_FILE = "LINK_FILE"
    LINK_DIRECTORY = "LINK_DIRECTORY"
    DIRECTORY = "DIRECTORY"
    FILE = "FILE"
    UNKNOWN = "UNKNOWN"

    @classmethod
    def from_pathlike(cls, f: Union[str, Path]):
        f = convert_to_path(f)
        if f.is_symlink():
            if f.is_file():
                return FileState.LINK_FILE
            elif f.is_dir():
                return FileState.LINK_DIRECTORY
            elif not f.exists():
                return FileState.BROKEN_LINK
        elif not f.exists():
            return FileState.MISSING
        elif f.is_file():
            return FileState.FILE
        elif f.is_dir():
            return FileState.DIRECTORY
        logger.error(f"Encountered a file in unknown state: {f}")
        return FileState.UNKNOWN



from enum import Flag
from pathlib import Path
from typing import Protocol


class LinkingResult(Flag):
    """The result from a linking operation"""

    SKIPPED = False
    CREATED = True
    FAILED = False


class LinkSpec(Protocol):
    src: Path
    dest: Path

    def __repr__(self) -> str:
        return "{} => {}".format(self.src, self.dest)

    def create(self, interactive: bool = False) -> LinkingResult:
        pass



from __future__ import annotations

import logging
import os
import shutil
from os import path
from pathlib import Path
from typing import NoReturn, Union, Optional, List, Tuple, Sequence

import pathspec  # type: ignore

import cfgcaddy
from cfgcaddy.file_state import FileState
from cfgcaddy.link_spec import LinkSpec, LinkingResult
from cfgcaddy.utils import (
    user_confirm,
    make_parent_dirs,
    create_dirs,
    convert_to_path,
    Pathlike,
)

logger = logging.getLogger()


# MyPy based exhaustiveness check
def assert_never(value: NoReturn) -> NoReturn:
    assert False, f"Unhandled value: {value} ({type(value).__name__})"


def create_symlink(location: Path, target: Path) -> None:
    make_parent_dirs(location)
    location.symlink_to(target, target_is_directory=target.is_dir())


class Link(LinkSpec):
    src: Path
    dest: Path

    def __init__(self, src: Union[str, Path], dest: Union[str, Path]):
        self.src = convert_to_path(src)
        self.dest = convert_to_path(dest)

    def create(self, interactive: bool = False) -> LinkingResult:
        dest_state = FileState.from_pathlike(self.dest)
        src_state = FileState.from_pathlike(self.src)
        if dest_state is FileState.UNKNOWN:
            logger.debug(
                f"Link destination {self.dest} is an unrecognized filetype, skipping"
            )
            return LinkingResult.SKIPPED

        if src_state is FileState.UNKNOWN:
            logger.debug(
                f"Link target {self.src} is an unrecognized filetype, skipping"
            )
            return LinkingResult.SKIPPED

        if dest_state in [FileState.LINK_FILE, FileState.LINK_DIRECTORY]:
            if self.dest.samefile(self.src):
                logger.debug(f"Skipping {self.dest}, already linked")
                return LinkingResult.SKIPPED
            else:
                logger.info(
                    f"{self.dest} is a symlink, please remove it if you want to link {self.src}"
                )
        if dest_state is FileState.BROKEN_LINK:
            if src_state in [
                FileState.FILE,
                FileState.DIRECTORY,
                FileState.LINK_FILE,
                FileState.LINK_DIRECTORY,
            ]:
                self.dest.unlink()
                create_symlink(self.dest, self.src)
            elif src_state in [FileState.MISSING, FileState.BROKEN_LINK]:
                return LinkingResult.SKIPPED
            else:
                assert_never(src_state)
        elif dest_state is FileState.FILE:
            if src_state in [FileState.FILE, FileState.DIRECTORY, FileState.LINK]:
                logger.info(
                    f"{self.dest} is a file, move/rename it if you want a link to {self.src}."
                )
                return LinkingResult.SKIPPED
            elif src_state is FileState.MISSING:
                logger.info(f"Copying {self.dest} to {self.src} and symlinking")
                shutil.move(self.dest, self.src)
                create_symlink(self.dest, self.src)
                return LinkingResult.CREATED
            elif src_state is FileState.BROKEN_LINK:
                self.src.unlink()
                logger.info(f"Copying {self.dest} to {self.src} and symlinking")
                shutil.move(self.dest, self.src)
                create_symlink(self.dest, self.src)
                return LinkingResult.CREATED
            else:
                assert_never(src_state)
        elif dest_state is FileState.DIRECTORY:
            logger.debug(f"Link destination {self.dest} does not exist")
            if src_state in [FileState.FILE, FileState.LINK_FILE]:
                logger.info(
                    f"{self.dest} is a file and the link src {self.src} is a file, skipping"
                )
                return LinkingResult.SKIPPED
            elif src_state is FileState.BROKEN_LINK:
                self.src.unlink()
                return link_folder(self.src, self.dest, interactive=interactive)
            elif src_state in [
                FileState.DIRECTORY,
                FileState.LINK_DIRECTORY,
                FileState.MISSING,
            ]:
                return link_folder(self.src, self.dest, interactive=interactive)
            else:
                assert_never(src_state)
        elif dest_state is FileState.MISSING:
            logger.debug(f"Link destination {self.dest} does not exist")
            if src_state in [FileState.FILE, FileState.LINK_FILE]:
                logger.debug(f"Link target {self.src} is a file, linking")
                create_symlink(self.dest, self.src)
                return LinkingResult.CREATED
            elif src_state in [FileState.DIRECTORY, FileState.LINK_DIRECTORY]:
                logger.debug(f"Link target {self.src} is a directory, linking")
                return link_folder(self.src, self.dest, interactive=interactive)
            elif src_state in [FileState.MISSING, FileState.BROKEN_LINK]:
                logger.error(
                    f"Trying to link {self.dest} to {self.src} both files do not exist"
                )
                return LinkingResult.SKIPPED
            else:
                assert_never(src_state)
        assert_never(dest_state)


def link_folder(src: Path, dest: Path, interactive: bool = False) -> LinkingResult:
    """Link the folder src to the destination dest

    Args:
        src (path) - The path to link from
        dest (path) - The path to link to

    Returns:
        True if an operation was performed, else False
    """
    folder_name = path.basename(src).strip(".")

    try:
        # Both Folders Exist
        if path.exists(dest) and path.exists(src) and not path.islink(dest):
            if not interactive or user_confirm(
                "Link and merge {} to {}".format(src, dest)
            ):
                absent_files, absent_dirs = find_absences(dest, src)
                zip_file = shutil.make_archive(
                    path.join(src, "{}_backup".format(folder_name)),
                    "zip",
                    root_dir=dest,
                )
                logger.info("Backing up {} to {}".format(folder_name, zip_file))
                create_dirs(absent_dirs)
                logger.info("Created {}".format(absent_dirs))
                move_files(links=absent_files)
                logger.info("Moving files {}".format(absent_files))
                shutil.rmtree(dest)
                logger.info("Removed {}".format(dest))
                create_symlink(dest, src)
                logger.info("Symlinked {} to {}".format(src, dest))
                return LinkingResult.CREATED

        # Only the source exists
        elif not path.exists(dest) and not path.islink(dest) and path.exists(src):
            try:
                os.makedirs(path.dirname(dest), exist_ok=True)
            except Exception as e:
                logger.error("Failed to make dir {} - {}".format(dest, e))
                return LinkingResult.FAILED
            create_symlink(dest, src)
            logger.info("Symlinked {} to {}".format(src, dest))
            return LinkingResult.CREATED

        # Only the destination exists
        elif path.exists(dest) and not path.exists(src):
            if not interactive or user_confirm(
                "Delete, Move to {} and" " Link back to {}?".format(src, dest)
            ):
                shutil.move(dest, src)
                logger.info("Moving {} to {}".format(dest, src))
                os.symlink(src, dest)
                logger.info("Symlinked {} to {}".format(src, dest))
                return LinkingResult.CREATED

        # Nothing to do
        logger.info(f"Skipping linking {src} to {dest}, not a supported usecase")
        return LinkingResult.SKIPPED

    except OSError as e:
        logger.error("Failed to link {} to {} - {}".format(src, dest, e))
        return LinkingResult.FAILED


def create_links(links: Optional[Sequence[LinkSpec]] = None) -> None:
    """Create symlinks for each item in links

    Args:
        links([Link]): a list of paths to link
    Returns:
        None: Does not return anything
    """
    if links:
        for link in links:
            try:
                make_parent_dirs(link.dest)
                os.symlink(link.src, link.dest)
            except OSError as err:
                # If the file exists and isn't a link
                if err.errno == 17 and not os.path.islink(link.dest):
                    logger.error(
                        "Can't make link from {} to {} because {}".format(
                            link.src, link.dest, err.strerror
                        )
                    )


def move_files(links: Sequence[LinkSpec]) -> None:
    """Move the files in the list from src to dest

    Args:
        links([Link])

    Returns:
        None: Does not return anything
    """
    if links:
        for link in links:
            try:
                shutil.move(link.src, link.dest)
            except OSError:
                logger.error("Can't move from {} to {}".format(link.src, link.dest))


def find_absences(
    src: Pathlike, dest: Pathlike, ignored_patterns: Optional[List[str]] = None
) -> Tuple[List[Link], List[str]]:
    """Walk the source directory and return a lists of files and dirs absent
        from the destination directory

    Args:
        src: The path to copy from (Default is the script's location)
        dest The path to copy to (Defaults to home directory)

    Returns:
        absent_files: a list of Links
        absent_dirs: a list of paths to directories
    """
    if not ignored_patterns:
        ignored_patterns = []
    absent_dirs = []
    absent_files = []

    default_ignore = ["!.*", cfgcaddy.DEFAULT_CONFIG_NAME, ".git"]

    ignored = pathspec.PathSpec.from_lines(
        "gitwildmatch", default_ignore + ignored_patterns
    )

    for root, dirs, files in os.walk(src, topdown=True):
        rel_path = path.relpath(root, src)
        if rel_path == ".":
            rel_path = ""

        dirs[:] = [
            dir_name
            for dir_name in dirs
            if not ignored.match_file(path.join(rel_path, dir_name))
        ]
        files[:] = [f for f in files if not ignored.match_file(path.join(rel_path, f))]

        # Create list of dirs that don't exist
        for dir_name in dirs:
            pathname = path.join(dest, rel_path, dir_name)
            if not path.exists(pathname):
                if path.islink(pathname) and not path.exists(pathname):
                    os.unlink(pathname)  # Fix Broken Links
                absent_dirs.append(pathname)

        # Create a list of files to be symlinked
        for f in files:
            pathname = path.join(dest, rel_path, f)
            if not path.exists(pathname):
                if path.islink(pathname) and not path.exists(pathname):
                    os.unlink(pathname)  # Fix Broken Links
                # Add the source and destination for the symlink
                absent_files.append(Link(path.join(root, f), pathname))
    return absent_files, absent_dirs



from pathlib import Path

from poetry_publish.publish import poetry_publish  # type: ignore

import cfgcaddy

PACKAGE_ROOT = Path(cfgcaddy.__file__).parent.parent


def publish():
    poetry_publish(
        package_root=PACKAGE_ROOT,
        version=cfgcaddy.__version__,
    )



import glob
import logging
import os
import platform
import sys
from os import path
from pathlib import Path
from typing import List, Dict

from ruamel.yaml import YAML

from cfgcaddy import utils
from cfgcaddy.link import Link

logger = logging.getLogger()
yaml = YAML(typ="safe")  # default, if not specified, is 'rt' (round-trip)


MISSING_FILE_MESSAGE = (
    "Please create a {} file in the same" " directory as the linker python file."
)


class LinkerConfig:
    config: Dict = {}
    links: List[Link] = []

    def __init__(self, config_file_path=None, default_config=None) -> None:
        self.config_file_path = config_file_path

        if default_config:
            self.config = default_config
        elif self.config_file_path:
            self.read_config()

        # Don't continue if we don't have a config
        if not self.config:
            logger.error("No config has been provided or loaded")
            sys.exit(1)

        if not self.linker_src or not self.linker_dest:
            logger.error("You need to specify a src and destination")
            sys.exit(1)

        self.generate_links(self.links_yaml)

    @property
    def preferences(self) -> Dict[str, str]:
        if not self.config:
            raise
        return self.config.get("preferences", {})

    @property
    def linker_src(self) -> Path:
        src = self.preferences.get("linker_src")

        if not src:
            raise ValueError("linker_src is not set in the config")
        return utils.expand_path(src)

    @property
    def linker_dest(self) -> Path:
        dest = self.preferences.get("linker_dest")
        if not dest:
            raise ValueError("linker_dest is not set in the config")
        return utils.expand_path(dest)

    def write_config(self, prompt=True) -> None:
        if not os.path.exists(self.config_file_path) or (
            not prompt
            or utils.user_confirm(
                "The file {} exists.\n"
                "Would you like to overwrite this file?".format(self.config_file_path)
            )
        ):
            try:
                logger.info("Writing config file")
                with open(self.config_file_path, "w") as file:
                    logger.debug(self.config_file_path)
                    yaml.dump(self.config, file)
            except Exception as e:
                logger.error("Error writing config: {}".format(e))

    def read_config(self) -> None:
        if os.path.isfile(self.config_file_path):
            with open(self.config_file_path, "r") as file:
                self.config = yaml.load(file)

    def generate_links(self, links):
        custom_links = []

        for link in links:
            try:
                if link.get("os") and platform.system() not in link.get("os"):
                    continue

                link_src = utils.expand_path(link["src"])
                link_destinations = link.get("dest")
                src_files = glob.glob(path.join(self.linker_src, link_src))

                # To account for no destination
                if not link_destinations:
                    if len(src_files) > 1:
                        link_destinations = [path.dirname(link_src)]
                    else:
                        link_destinations = [link_src]
                for src_path in src_files:
                    for dest in map(utils.expand_path, link_destinations):
                        if len(src_files) > 1:
                            src_name = path.join(dest, path.basename(src_path))
                        else:
                            src_name = dest
                        if path.isabs(dest):
                            dest_path = src_name
                        else:
                            dest_path = path.join(self.linker_dest, src_name)
                        custom_links.append(
                            Link(
                                utils.expand_path(src_path),
                                utils.expand_path(dest_path),
                            )
                        )
            except KeyError:
                logger.exception("Bad custom link")

        logger.debug("Custom Links => {}".format(custom_links))
        self.links = custom_links

    @property
    def links_yaml(self):
        """Parse the YAML config representation into a consistent format"""
        links = []
        for link in self.config.get("links", []):
            if type(link) is str:
                link = {"src": link, "dest": []}
            if type(link.get("dest")) is str:
                link["dest"] = [link["dest"]]
            links.append(link)
        logger.debug("Links before formatting: {}".format(self.config.get("links")))
        logger.debug("Links after formatting: {}".format(links))
        return links

    @property
    def ignore_patterns(self):
        lines = self.config.get("ignore", [])
        """Parse the gitignore style file
        """
        logger.debug("Ignore Patterns: {}".format(lines))
        return lines
